package ru.bitel.bgbilling.modules.sorm.dyn.cdr;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import ru.bitel.bgbilling.common.BGException;
import ru.bitel.bgbilling.modules.sorm.common.SormManufacturer;
import ru.bitel.bgbilling.modules.sorm.common.bean.SormProject;
import ru.bitel.bgbilling.modules.sorm.server.upload.cdr.CDRItem;
import ru.bitel.bgbilling.modules.sorm.server.upload.cdr.voice.VoiceCDRUploadCommon;
import ru.bitel.bgbilling.modules.sorm.server.upload.norsitrans.NorsiTransFileTitle;
import ru.bitel.common.TimeUtils;
import ru.bitel.common.Utils;

import java.nio.file.Path;
import java.util.Arrays;

import static ru.bitel.bgbilling.modules.sorm.server.upload.norsitrans.NorsiTransConstants.FILE_TITLE_PATTERN;
import static ru.bitel.bgbilling.modules.sorm.server.utils.SormUtils.getUploadFile;
/*
Для АТС MD110
I - входящий из ТфОП
M - Исходящий в ТфОП
J - вызов внутри коммутатора
T - трансфер звонка
NI - переадресация

Дата и время, продолжительность, номер А, номер Б
17.04.2025 12:40:38, 22, 4112, 84955804935, , , M, , 0   31, , , , 00 010101
17.04.2025 12:40:52, 8, 9858967933, 3811, , , I, , , , , , 00
17.04.2025 12:41:24, 36, 3806, 7194969, , , M, , 0   31, , , , 00 010102
17.04.2025 12:48:53, 7, 4110, 3811, , , J, , , , , , 00
17.04.2025 12:48:13, 47, 9166165903, 9861, , , I, , , , , , 00
 */
public class NorsiTransVoiceCDRUpload
    extends VoiceCDRUploadCommon
{
    private static final Logger log = LogManager.getLogger();

    public NorsiTransVoiceCDRUpload( SormProject sormProject, String cdrNumberConverters )
        throws BGException
    {
        super( sormProject, cdrNumberConverters );
    }

    @Override
    public SormManufacturer getSormManufacturer()
    {
        return SormManufacturer.NORSITRANS;
    }

    @Override
    public Path getFile()
        throws BGException
    {
        return getUploadFile( localUserPath,
                              NorsiTransFileTitle.CDR.getTitle(),
                              FILE_TITLE_PATTERN );
    }

    @Override
    public String[] getCDRItem( CDRItem cdrItem )
        throws BGException
    {
        String[] raw = cdrItem.otherInfo;

        log.debug( "Новая запись для сорма raw = " + Arrays.toString(raw) );

        String callingE164 = applyCallingNumberConverterRule( cdrItem.callingNumber );
        String calledE164 = applyCalledNumberConverterRule( cdrItem.calledNumber );
        String conditionCode = raw[5];
        String externalLineId = raw[8];

        try
        {
        /*
            Разделитель – табуляция.
            Поля не заключаются в экранирующие кавычки.
         */
            String[] item = new String[17];
            item[0] = regionId; //идентификатор оператора связи или филиала (регион) (число);
            item[1] = TimeUtils.formatFullDate( cdrItem.startTime ); //дата и время начала соединения;
            item[2] = String.valueOf( cdrItem.duration ); //длительность, сек (число);
            /*
            поле 4 - по справочнику, типа такого:
            "1";"0";"01.01.1999 00:00:00";"";"Телефонный вызов"
            "1";"1";"01.01.1999 00:00:00";"";"Исходящий вызов на ТФОП"
            "1";"2";"01.01.1999 00:00:00";"";"Входящий вызов из ТФОП"
            "1";"3";"01.01.1999 00:00:00";"";"Внутренний вызов в пределах сети АО ОператорСвязи"
             */
            String type = "0";
            if( "M".equals( conditionCode ) )
            {
                type = "1";
            }
            else if( "I".equals( conditionCode ) )
            {
                type = "2";
            }
            else if( "J".equals( conditionCode ) )
            {
                type = "3";
            }
            item[3] = type; //тип вызова (справочник типов вызовов) (число);

            item[4] = getServiceId(); //ДВО (справочник список ДВО) (число);
            item[5] = isOurNumber( callingE164 ) ? "0" : "1"; //тип вызывающего абонента (0 - абонент данного коммутатора, 1 - абонент сети связи);
            item[6] = isOurNumber( calledE164 ) ? "0" : "1"; //тип вызываемого абонента (0 - абонент данного коммутатора, 1 - абонент сети связи);

            item[7] = String.valueOf( cdrItem.deviceId ); //код АТС/коммутатора/MGW обслужившего вызов (код по справочнику коммутатора, обслужившего соединение) (строка);
            item[8] = "0101"; //входящая транковая группа (код по справочнику пучков соединительных линий) (число); // всегда значение 0101 для данного оператора

            String outBunch = "";
            if ( "M".equals( conditionCode ) && externalLineId != null && externalLineId.trim().length() >= 4)
            {
                outBunch = externalLineId.trim().substring(0, 4);
            }
            item[9] = Utils.maskBlank( outBunch, "" ); //исходящая транковая группа (код по справочнику пучков соединительных линий) (число);

            item[10] = "607"; //причина разъединения соединения (код по справочнику причин завершения соединений) (число);

            /*
            item[11] = conditionCode.equals( "I" ) ? cdrItem.callingNumber : callingNumber; //идентификатор вызывающего абонента (Абонент - А) (строка); (поле "Calling number" из сырого лога при входящих вызовах из ТФОП, при исходящих вызовах со станции номер в формате E.164, преобразовнный из поля "Calling number";поле 1)
            // если звонящий не наш абонент, то поле пустое
            item[12] = conditionCode.equals( "O" ) ? cdrItem.callingNumber : ""; //внутренний номер вызывающего абонента (Абонент А) (строка);
            item[13] = cdrItem.calledNumber; //набранный номер вызываемого абонента (набранный Абонентом А номер для вызова Абонента Б) (строка);
            item[14] = conditionCode.equals( "O" ) ? cdrItem.calledNumber : calledNumber; //идентификатор вызываемого абонента (Абонент - Б) (строка);
            item[15] = cdrItem.calledNumber; //внутренний номер вызываемого абонента (Абонент Б) (строка);
             */
            // вынес в метод парсинг этих 5 полей
            parseIdentifiers( item, cdrItem.callingNumber, cdrItem.calledNumber, callingE164, calledE164, conditionCode );

            //17 - поле "External Line ID" из сырого лога при "Condition Code" = NI.
            if( "NI".equals( conditionCode ) )
            {
                item[16] = Utils.maskBlank( externalLineId, "" ); //телефонный номер при переадресации (строка).
            }
            else
            {
                item[16] = ""; //телефонный номер при переадресации (строка).
            }

            return item;
        }
        catch( Exception e )
        {
            throw new BGException( e );
        }
    }

    public void parseIdentifiers( String[] item, String calling, String called, String callingE164, String calledE164, String conditionCode )
    {
        String field11 = "";  // Calling number (при входящих из ТФОП или E.164 при исходящих)
        String field12 = "";  // Calling number (при исходящих со станции)
        String field13 = called;  // Dialled number (всегда)
        String field14 = "";  // Dialled number в E.164 (при всех, кроме входящего на ТФОП)
        String field15 = "";  // Dialled number при входящих из ТФОП и внутр. вызовах (не на ТФОП)

        boolean isIncoming = "I".equals( conditionCode );
        boolean isOutgoing = "M".equals( conditionCode );
        boolean isInternal = "J".equals( conditionCode );

        if ( isIncoming )
        {
            field11 = calling;               // Входящий — берём как есть
            field14 = calledE164; // Преобразуем Dialled в E.164
            field15 = called;              // Dialled используется
        }
        else if ( isOutgoing || isInternal )
        {
            field11 = callingE164; // Преобразуем Calling в E.164
            field12 = calling;               // Оригинальный номер
            if ( called.isEmpty() )
            {
                field14 = "";                      // Исходящий на ТФОП — dialled пустой
                field15 = "";                      // Исходящий на ТФОП — тоже пустой
            }
        }

        item[11] = Utils.maskBlank( field11, "" );
        item[12] = Utils.maskBlank( field12, "" );
        item[13] = Utils.maskBlank( field13, "" );
        item[14] = Utils.maskBlank( field14, "" );
        item[15] = Utils.maskBlank( field15, "" );
    }

    @Override
    public String[] getCDRItem( String[] cdrFields )
        throws BGException
    {
        return null;
    }

    @Override
    public String[] getHeader()
    {
        return null;
    }
}